/**
 * Copyright (c) 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.probes.api;

import java.util.Iterator;
import java.util.ServiceLoader;

import org.ow2.petals.probes.api.exceptions.MultipleProbesFactoriesFoundException;
import org.ow2.petals.probes.api.exceptions.NoProbesFactoryFoundException;
import org.ow2.petals.probes.api.key.ProbeKey;

/**
 * Builds the keyed probes factory.
 * 
 * @param <K>
 *            Type of the key identifying each counter, implementing
 *            {@link ProbeKey}.
 * @param <T>
 *            The type of measured values (internal value type).
 * @param <S>
 *            The type of value to return to client probe (external value type).
 * 
 * @author Christophe DENEUX (Linagora)
 */
public class KeyedProbesFactoryBuilder<K extends ProbeKey, T extends Comparable<T>, S> {

    /**
     * <p>
     * Get the {@link KeyedProbesFactory} installed.
     * </p>
     * <p>
     * If the system property
     * <code>org.ow2.petals.probes.api.KeyedProbesFactory</code> is defined and
     * set, its value will be used as probes factory, otherwise, the probes
     * factory will be loaded from the service
     * <code>org.ow2.petals.probes.api.KeyedProbesFactory</code>
     * </p>
     * 
     * @throws MultipleProbesFactoriesFoundException
     *             Several probes factories are found.
     * @throws NoProbesFactoryFoundException
     *             No probes factory is found.
     */
    public KeyedProbesFactory<K, T, S> getKeyedProbesFactory()
            throws MultipleProbesFactoriesFoundException,
            NoProbesFactoryFoundException {
        final ServiceLoader<KeyedProbesFactory> probesFactoryLoader = ServiceLoader
                .load(KeyedProbesFactory.class);

        final Iterator<KeyedProbesFactory> probesFactoryIterator = probesFactoryLoader.iterator();
        if (probesFactoryIterator.hasNext()) {
            final KeyedProbesFactory probesFactory = probesFactoryIterator.next();
            if (probesFactoryIterator.hasNext()) {
                throw new MultipleProbesFactoriesFoundException();
            }
            return probesFactory;
        } else {
            throw new NoProbesFactoryFoundException();
        }
    }
    
    /**
     * Get a specific {@link KeyedProbesFactory} installed.
     * 
     * @throws NoProbesFactoryFoundException
     *             The probes factory is not found.
     */
    public KeyedProbesFactory<K, T, S> getKeyedProbesFactory(final String probesFactoryName)
            throws NoProbesFactoryFoundException {
        final ServiceLoader<KeyedProbesFactory> probesFactoryLoader = ServiceLoader
                .load(KeyedProbesFactory.class);

        final Iterator<KeyedProbesFactory> probesFactoryIterator = probesFactoryLoader.iterator();
        while (probesFactoryIterator.hasNext()) {
            final KeyedProbesFactory probesFactory = probesFactoryIterator.next();
            if (probesFactory.getClass().getName().equals(probesFactoryName)) {
                return probesFactory;
            }
        }
        // No probes factory matching the provided name was found.
        throw new NoProbesFactoryFoundException();
    }

}
