/**
 * Copyright (c) 2013-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.probes.api;

import java.util.Iterator;
import java.util.ServiceLoader;

import org.ow2.petals.probes.api.exceptions.MultipleProbesFactoriesFoundException;
import org.ow2.petals.probes.api.exceptions.NoProbesFactoryFoundException;

/**
 * Builds the macro probes factory.
 * 
 * @author Christophe DENEUX (Linagora)
 */
public class MacroProbesFactoryBuilder {

    /**
     * <p>
     * Get the {@link MacroProbesFactory} installed.
     * </p>
     * <p>
     * If the system property
     * <code>org.ow2.petals.probes.api.MacroProbesFactory</code> is defined and
     * set, its value will be used as probes factory, otherwise, the probes
     * factory will be loaded from the service
     * <code>org.ow2.petals.probes.api.MacroProbesFactory</code>
     * </p>
     * 
     * @throws MultipleProbesFactoriesFoundException
     *             Several probes factories are found.
     * @throws NoProbesFactoryFoundException
     *             No probes factory is found.
     */
    public MacroProbesFactory getMacroProbesFactory() throws MultipleProbesFactoriesFoundException,
            NoProbesFactoryFoundException {
        final ServiceLoader<MacroProbesFactory> probesFactoryLoader = ServiceLoader
                .load(MacroProbesFactory.class);

        final Iterator<MacroProbesFactory> probesFactoryIterator = probesFactoryLoader.iterator();
        if (probesFactoryIterator.hasNext()) {
            final MacroProbesFactory probesFactory = probesFactoryIterator.next();
            if (probesFactoryIterator.hasNext()) {
                throw new MultipleProbesFactoriesFoundException();
            }
            return probesFactory;
        } else {
            throw new NoProbesFactoryFoundException();
        }
    }
    
    /**
     * Get a specific {@link MacroProbesFactory} installed.
     * 
     * @throws NoProbesFactoryFoundException
     *             The probes factory is not found.
     */
    public MacroProbesFactory getMacroProbesFactory(final String probesFactoryName)
            throws NoProbesFactoryFoundException {
        final ServiceLoader<MacroProbesFactory> probesFactoryLoader = ServiceLoader
                .load(MacroProbesFactory.class);

        final Iterator<MacroProbesFactory> probesFactoryIterator = probesFactoryLoader.iterator();
        while (probesFactoryIterator.hasNext()) {
            final MacroProbesFactory probesFactory = probesFactoryIterator.next();
            if (probesFactory.getClass().getName().equals(probesFactoryName)) {
                return probesFactory;
            }
        }
        // No probes factory matching the provided name was found.
        throw new NoProbesFactoryFoundException();
    }

}
