/**
 * Copyright (c) 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.probes.api;

import java.util.Timer;

import org.ow2.petals.probes.api.probes.CounterGaugeProbe;
import org.ow2.petals.probes.api.probes.CounterProbe;
import org.ow2.petals.probes.api.probes.DurationProbe;
import org.ow2.petals.probes.api.probes.GaugeProbe;
import org.ow2.petals.probes.api.sensor.GaugeSensor;
import org.ow2.petals.probes.api.sensor.detector.GaugeDefectDetector;

/**
 * Probe factory.
 * 
 * @param <T>
 *            The type of measured values (internal value type).
 * @param <S>
 *            The type of value to return to client probe (external value type).
 * 
 * @author Christophe DENEUX (Linagora)
 */
public interface ProbesFactory<T extends Comparable<T>, S> {

    /**
     * Create a new counter probe instance.
     * 
     * @see CounterProbe
     */
    public CounterProbe createCounterProbe();

    /**
     * Create a new counter gauge probe instance.
     * 
     * @param initialValue
     *            Initial value of the counter gauge.
     * @see CounterGaugeProbe
     */
    public CounterGaugeProbe createCounterGaugeProbe(final long initialValue);

    /**
     * Create a new gauge probe instance using a specific sensor.
     * 
     * @param sensor
     *            The sensor of the probe
     * 
     * @see GaugeProbe
     */
    public GaugeProbe<T, S> createGaugeProbe(final GaugeSensor<T, S> sensor);

    /**
     * Create a new gauge probe instance using a specific sensor and a defect
     * detector.
     * 
     * @param sensor
     *            The sensor of the probe
     * @param defectDetector
     *            The defect detector associated to the sensor.
     * 
     * @see GaugeProbe
     */
    public GaugeProbe<T, S> createGaugeProbe(final GaugeSensor<T, S> sensor,
            final GaugeDefectDetector<T> defectDetector);

    /**
     * Create a new duration probe instance, where durations are expressed in <b>milliseconds</b>.
     * 
     * @param timer
     *            The {@link Timer} running the samples switches task
     * @param samplePeriod
     *            Duration between two samples switches
     * @throws IllegalArgumentException
     *             <code>timer</code> is <code>null</code>, or <code>samplePeriod</code> has an invalid value.
     * @see DurationProbe
     */
    public DurationProbe createDurationProbe(final Timer timer, final long samplePeriod)
            throws IllegalArgumentException;

    /**
     * Create a new duration probe instance, where durations are expressed in <b>nanoseconds</b>.
     * 
     * @param timer
     *            The {@link Timer} running the samples switches task
     * @param samplePeriod
     *            Duration between two samples switches
     * @throws IllegalArgumentException
     *             <code>timer</code> is <code>null</code>, or <code>samplePeriod</code> has an invalid value.
     * @see DurationProbe
     */
    public DurationProbe createDurationNanoProbe(final Timer timer, final long samplePeriod)
            throws IllegalArgumentException;

}
