/**
 * Copyright (c) 2012 EBMWebsourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.probes.api.probes;

/**
 * <p>
 * Absolute response time values, composed of:
 * </p>
 * <ul>
 * <li>the max response time,</li>
 * <li>the avg response time,</li>
 * <li>the min response time.</li>
 * </ul>
 * <p>
 * For resources consumption, the avg response time is computed from the sum of
 * each response time and the number of response times, instead of keeping each
 * response time in memory.
 * </p>
 * 
 * @author Christophe DENEUX (PetalsLink)
 * 
 */
public class AbsoluteResponseTimes {

    /**
     * The sum of response times to compute the avg response time with
     * {@link #responseTimesNb}.
     */
    // TODO: What to do when the sum will exceed the max long value ?
    protected long responseTimesSum;

    /**
     * The number of response times to compute the avg response time with
     * {@link #responseTimesSum}.
     */
    // TODO: What to do when the sum will exceed the max long value ?
    protected long responseTimesNb;

    /**
     * The max response time
     */
    protected long maxResponseTime;

    /**
     * The min resposne time
     */
    protected long minResponseTime;

    /**
     * Creates absolute response time values from relative response time values
     * ({@link ResponseTimeRelativeValues}.
     * 
     * @param value
     *            The relative response time values.
     */
    public AbsoluteResponseTimes(final ResponseTimeRelativeValues value) {
        this.maxResponseTime = value.getMax();
        this.minResponseTime = value.getMin();
        this.responseTimesNb = value.getResponseTimesNb();
        this.responseTimesSum = value.getResponseTimesSum();
    }

    /**
     * @return The max response time values
     */
    public long getMaxResponseTime() {
        return this.maxResponseTime;
    }

    /**
     * @return The min response time values
     */
    public long getMinResponseTime() {
        return this.minResponseTime;
    }

    /**
     * @return The avg response time values
     */
    public long getAvgResponseTime() {
        return this.responseTimesSum / this.responseTimesNb;
    }

    /**
     * @return The sum of response times to compute the avg response time
     */
    public long getResponseTimesSum() {
        return this.responseTimesSum;
    }

    /**
     * @return The number of response times to compute the avg response time
     */
    public long getResponseTimesNb() {
        return this.responseTimesNb;
    }

    /**
     * Update absolute response time values from relative response time values.
     * 
     * @param responseTimeRelativeValues
     *            Relative response time values
     */
    public void update(final ResponseTimeRelativeValues responseTimeRelativeValues) {
        // Update the max response timer
        final long maxRelativeResponseTime = responseTimeRelativeValues.getMax();
        if (this.maxResponseTime < maxRelativeResponseTime) {
            this.maxResponseTime = maxRelativeResponseTime;
        }

        // Update the min response timer
        final long minRelativeResponseTime = responseTimeRelativeValues.getMin();
        if (this.minResponseTime > minRelativeResponseTime) {
            this.minResponseTime = minRelativeResponseTime;
        }

        // Update the avg response time
        this.responseTimesSum += responseTimeRelativeValues.getResponseTimesSum();
        this.responseTimesNb += responseTimeRelativeValues.getResponseTimesNb();
    }

}
