/**
 * Copyright (c) 2012 EBMWebsourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.probes.api.probes;

import java.util.Map;

import org.ow2.petals.probes.api.exceptions.ProbeKeyMissingException;
import org.ow2.petals.probes.api.exceptions.ProbeNotInitializedException;
import org.ow2.petals.probes.api.exceptions.ProbeNotStartedException;
import org.ow2.petals.probes.api.key.ProbeKey;

/**
 * <p>
 * A counter probe to count the number of something by key (K) since its last
 * start.
 * </p>
 * <p>
 * <u><b>Lifecycle</b>:</u> The counter values are incremented only if the probe
 * is started. Counter values can be acceded even if the probe is initialized or
 * stopped. Counter values are reset when the probe starts or shutdowns.
 * </p>
 * 
 * @param <K>
 *            Type of the key identifying each counter, implementing
 *            {@link ProbeKey}.
 * 
 * @author Christophe DENEUX (PetalsLink)
 * 
 */
public interface KeyedCounterProbe<K extends ProbeKey> extends Probe {

    /**
     * Increments the value of the counter.
     * 
     * @throws ProbeNotStartedException
     *             The probe is not started.
     */
    public void inc(final K key) throws ProbeNotStartedException;

    /**
     * Accessor to the counter values.
     * 
     * @return A copy of the counter values since the last probe start.
     * @throws ProbeNotInitializedException
     *             The probe is not initialized.
     */
    public Map<K, Long> getValues() throws ProbeNotInitializedException;

    /**
     * Accessor to the counter values as simple types.
     * 
     * @return A copy of the counter values, with simple types, since the last
     *         probe start.
     * @throws ProbeNotInitializedException
     *             The probe is not initialized.
     */
    public Map<String[], Long> getConvertedValues() throws ProbeNotInitializedException;

    /**
     * Decrements the counter associated to the key <code>originalKey</code> and
     * increments the counter associated to the key <code>newKey</code>.
     * 
     * @param originalKey
     *            The key associated to the counter to decrement
     * @param newKey
     *            The key associated to the counter to increment
     * @throws ProbeNotStartedException
     *             The probe is not started.
     * @throws ProbeKeyMissingException
     *             The <code>originalKey</code>, or <code>newKey</code> does not
     *             exist in the probe.
     */
    public void move(final K originalKey, final K newKey) throws ProbeKeyMissingException,
            ProbeNotStartedException;
}
