/**
 * Copyright (c) 2012 EBMWebsourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.probes.api.probes;

import java.util.Map;

import org.ow2.petals.probes.api.exceptions.ProbeNotInitializedException;
import org.ow2.petals.probes.api.exceptions.ProbeNotStartedException;
import org.ow2.petals.probes.api.key.ProbeKey;

/**
 * <p>
 * A gauge probe to probe instant value, by key (K), on demand, and storing the
 * max reached value (that can be greater than the max of instant values) since
 * its last start.
 * </p>
 * <p>
 * <u><b>Lifecycle</b>:</u> The instant values can be acceded or picked only if
 * the probe is started. The max values can be acceded even if the probe is
 * initialized or stopped. The max values are reseted to the sensor initial
 * value when the probe starts.
 * </p>
 * 
 * @param <K>
 *            Type of the key identifying each value, implementing
 *            {@link ProbeKey}.
 * @param <T>
 *            The type of measured values (internal value type).
 * @param <S>
 *            The type of value to return to client probe (external value type).
 * 
 * @author Christophe DENEUX (PetalsLink)
 * 
 */
public interface KeyedGaugeProbe<K extends ProbeKey, T extends Comparable<T>, S> extends Probe {

	/**
	 * <p>
	 * Realizes a measure and compare the value to the max one to update it.
	 * </p>
	 * <p>
	 * This method <b>MUST</b> be invoked by the application internals when the
	 * resource associated to the probe changes.
	 * </p>
	 * <p>
	 * This method is thread-safe
	 * </p>
	 * 
	 * @param key
	 *            The key for which the measure must be done.
	 * @throws ProbeNotStartedException
	 *             The probe is not started.
	 */
    public void pick(final K key) throws ProbeNotStartedException;

	/**
	 * <p>
	 * Accessor to the sensor instant values of all keys.
	 * </p>
	 * <p>
	 * This method <b>MUST</b> be invoked by monitoring clients to get the probe
	 * instant values.
	 * </p>
	 * <p>
	 * This method is thread-safe
	 * </p>
	 * 
	 * @return A copy of the sensor instant values associated to all keys intercepted
	 *         since the last probe start.
	 * @throws ProbeNotStartedException
	 *             The probe is not started.
	 */
    public Map<K, T> getInstantValues() throws ProbeNotStartedException;
	
    /**
     * <p>
     * Accessor to the sensor instant values of all keys, as simple types.
     * </p>
     * <p>
     * This method <b>MUST</b> be invoked by monitoring clients to get the probe
     * instant values.
     * </p>
     * <p>
     * This method is thread-safe
     * </p>
     * 
     * @return A copy of the sensor instant values associated to all keys
     *         intercepted since the last probe start.
     * @throws ProbeNotStartedException
     *             The probe is not started.
     */
    public Map<String[], S> getConvertedInstantValues() throws ProbeNotStartedException;

	/**
	 * <p>
	 * Accessor to the max value.
	 * </p>
	 * <p>
	 * This method is thread-safe
	 * </p>
	 * 
	 * @return The probe max value.
	 * @throws ProbeNotInitializedException
	 *             The probe is not initialized.
	 */
    public Map<K, T> getMaxValues() throws ProbeNotInitializedException;
	
    public Map<String[], S> getConvertedMaxValues() throws ProbeNotInitializedException;
}
