/**
 * Copyright (c) 2012 EBMWebsourcing, 2012-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.probes.api.probes;

import java.util.Map;

import org.ow2.petals.probes.api.enums.ExecutionStatus;
import org.ow2.petals.probes.api.exceptions.ProbeNotInitializedException;
import org.ow2.petals.probes.api.exceptions.ProbeNotStartedException;
import org.ow2.petals.probes.api.exceptions.ResponseTimeCollectionStoppedException;
import org.ow2.petals.probes.api.exceptions.StartDateItemLostException;
import org.ow2.petals.probes.api.exceptions.StartDateItemUnknownException;
import org.ow2.petals.probes.api.key.ExecutionStatusProbeKey;
import org.ow2.petals.probes.api.key.ProbeKey;

/**
 * <p>
 * A response time probe to probe periodically response times, by key (K), and
 * storing:
 * </p>
 * <ul>
 * <li>as absolute values:
 * <ul>
 * <li>the max response time,</li>
 * <li>the avg response time,</li>
 * <li>the min response time,</li>
 * </ul>
 * </li>
 * <li>as relatives values:
 * <ul>
 * <li>the max response time,</li>
 * <li>the avg response time,</li>
 * <li>the min response time,</li>
 * <li>the 10-percentile response time: 10% of the response times are lesser
 * than this value,</li>
 * <li>the 50-percentile response time: 50% of the response times are lesser
 * than this value,</li>
 * <li>the 90-percentile response time: 90% of the response times are upper than
 * this value.</li>
 * </ul>
 * </li>
 * </ul>
 * <p>
 * When the end of a sample occurs, relative response time values are computed
 * with the response times collected during the sample, and the absolute
 * response time values are updated.
 * </p>
 * <p>
 * <u><b>Lifecycle</b>:</u>The relative response time values can be acceded only
 * if the probe is started. The absolute response time values can be acceded
 * even if the probe is stopped. The collection of response times starts when
 * the probe starts. The absolute and relative response time values are reseted
 * when the probe starts.
 * </p>
 * 
 * @param <K>
 *            Type of the key identifying each response time values,
 *            implementing {@link ProbeKey}.
 * 
 * @author Christophe DENEUX (PetalsLink)
 * 
 */
public interface KeyedResponseTimeProbe<K extends ProbeKey> extends Probe {

    /**
     * A new execution starts, invoke this method to pick the current time. When
     * the execution will end, use
     * {@link #endsExecution(KeyedStartDateItem, ExecutionStatus)} to close the
     * execution and compute the response time.
     * 
     * @param key
     *            The key for which the measure must be done.
     * @throws ProbeNotStartedException
     *             The probe is not started.
     * @throws ResponseTimeCollectionStoppedException
     *             A sample switch is in progress, no response time collection
     *             is available. This exception should not occur.
     * @throws StartDateItemLostException
     *             Unable to store the start time of the new execution. This
     *             exception should not occur.
     * @return A {@link KeyedStartDateItem} created for this new execution.
     */
    public KeyedStartDateItem<K> newExecution(final K key) throws ProbeNotStartedException,
            ResponseTimeCollectionStoppedException, StartDateItemLostException;

    /**
     * Ends an execution and compute the response time.
     * 
     * @see #newExecution(ProbeKey)
     * @param startDateItem
     *            The {@link KeyedStartDateItem} created by
     *            {@link #newExecution(ProbeKey)} for the current execution.
     * @param executionStatus
     *            The execution status of this execution
     * @throws ProbeNotStartedException
     *             The probe is not started.
     * @throws ResponseTimeCollectionStoppedException
     * @throws StartDateItemUnknownException
     *             <code>startDateItem</code> not found in the list of start
     *             dates. This exception should not occur.
     */
    public void endsExecution(final KeyedStartDateItem<K> startDateItem,
            final ExecutionStatus executionStatus) throws ProbeNotStartedException,
            ResponseTimeCollectionStoppedException, StartDateItemUnknownException;

    /**
     * @return The absolute response time values.
     * @throws ProbeNotInitializedException
     *             The probe is not initialized or shutdown.
     */
    public Map<ExecutionStatusProbeKey<K>, AbsoluteResponseTimes> getAbsoluteResponseTimeValues()
            throws ProbeNotInitializedException;

    /**
     * @return The absolute response time values converted into simple java
     *         types. Each key of the map entries is an array of {@link String}
     *         values such as:
     *         <ol>
     *         <li>the first ones are the grouping key of this probe,</li>
     *         <li>the last one is the execution status of the requests.</li>
     *         </ol>
     *         Each value of the map entries is an array of long values such as:
     *         <ol>
     *         <li>the first long value is the max response time,</li>
     *         <li>the second long value is the average of response times,</li>
     *         <li>the third long value is the min response time.</li>
     *         </ol>
     * @throws ProbeNotInitializedException
     *             The probe is not initialized or shutdown.
     */
    public Map<String[], Long[]> getConvertedAbsoluteResponseTimeValues()
            throws ProbeNotInitializedException;

    /**
     * @return The relative response time values.
     * @throws ProbeNotStartedException
     *             The probe is not started or stopped.
     */
    public Map<ExecutionStatusProbeKey<K>, ResponseTimeRelativeValues> getRelativeResponseTimeValues()
            throws ProbeNotStartedException;

    /**
     * @return The relative response time values converted into simple java
     *         types. Each value is an array of long values such as:
     *         <ol>
     *         <li>the first long value is the max response time,</li>
     *         <li>the second long value is the average of response times,</li>
     *         <li>the third long value is the min response time,</li>
     *         <li>the fourth long value is the 10-percentile response time,</li>
     *         <li>the fourth long value is the 50-percentile response time,</li>
     *         <li>the fourth long value is the 90-percentile response time.</li>
     *         </ol>
     * @throws ProbeNotStartedException
     *             The probe is not started or stopped.
     */
    public Map<String[], Long[]> getConvertedRelativeResponseTimeValues()
            throws ProbeNotStartedException;
}
